==> apgl-0.6.1.tar.gz: apgl-0.6.1/apgl/graph/AbstractSingleGraph.py <==
'''
Created on 30 Jun 2009

@author: charanpal
'''

import numpy 
from apgl.util.Util import Util
from apgl.graph.AbstractGraph import AbstractGraph

class AbstractSingleGraph(AbstractGraph):
    """
    A basic abstract base class for graphs. An AbstractSingleGraph can only have
    a single edge at most between any two vertices, but edges and vertices can be
    labelled with any object. Verticies are indexed with integer values.
    """
    def __init__(self):
        pass

    def addEdge(self, vertexIndex1, vertexIndex2, edgeValue):
        """
        Add a non-zero edge between two vertices.
        
        :param vertexIndex1: The index of the first vertex.
        :type vertexIndex1: :class:`int`

        :param vertexIndex2: The index of the second vertex.
        :type vertexIndex2: :class:`int`

        :param edgeValue: The value to assign to the edge.
        """

        Util.abstract()

    def addEdges(self, edgeIndexArray, edgeValues=[]):
        """
        Takes a numpy array of edge index pairs, and edge values and adds them
        to this graph. The array is 2 dimensional such that each row is a pair
        of edge indices.

        :param edgeIndexArray: The array of edge indices with each being a pair of indices. 
        :type edgeIndexArray: :class:`numpy.ndarray`

        :param edgeValues: The list of edge values
        :type edgeValues: :class:`list`
        """

        if edgeValues != []: 
            for i in range(edgeIndexArray.shape[0]):
                self.addEdge(edgeIndexArray[i, 0], edgeIndexArray[i, 1], edgeValues[i])
        else:
            for i in range(edgeIndexArray.shape[0]):
                self.addEdge(edgeIndexArray[i, 0], edgeIndexArray[i, 1], 1)
    
    def removeEdge(self, vertexIndex1, vertexIndex2):
        """
        Remove an edge between two vertices.

        :param vertexIndex1: The index of the first vertex.
        :type vertexIndex1: :class:`int`

        :param vertexIndex2: The index of the second vertex.
        :type vertexIndex2: :class:`int`
        """
        Util.abstract()
    
    def getNumEdges(self):
        """ Returns the total number of edges in the graph. """
        Util.abstract()
    
    def getNumVertices(self):
        """ Returns the total number of vertices in the graph. """
        Util.abstract()
    
    def isUndirected(self):
        """ Returns true if the current graph is undirected, otherwise false. """
        Util.abstract()
    
    def neighbours(self, vertexIndex1):
        """ Return a iterable item of neighbours (indices) """
        Util.abstract()

    def getEdge(self, vertexIndex1, vertexIndex2):
        """ Return an edge between two vertices """
        Util.abstract()

    def getVertex(self, vertexIndex):
        """ Return a vertex of given index """
        Util.abstract()

    def getVertices(self, vertexIndices):
        """
        Takes a list of vertex indices and returns the corresponding vertex values. 
        """
        vertices = [] 

        for vertexIndex in vertexIndices:
            vertices.append(self.getVertex(vertexIndex))

        return vertices 

    def getAllVertexIds(self):
        """
        Return a list of all indices of the vertices
        
        :returns:  A numpy array of all the vertex indices in this graph. 
        """

        Util.abstract()

    def setVertices(self, vertexIndices, vertices):
        if len(vertexIndices) != len(vertices):
            raise ValueError("Must be lists of same length")

        for i in range(len(vertexIndices)):
            self.setVertex(vertexIndices[i], vertices[i])

    def setVertex(self, vertexIndex, vertex):
        """ Assign a value to a vertex """
        Util.abstract()

    def getAllEdges(self):
        """
        Return an array of edges with each row representing an edge.

        :returns:  A numpy array of all edges in this graph. 
        """
        Util.abstract()

    def toNetworkXGraph(self):
        """
        Convert this graph into a networkx Graph or DiGraph object, which requires 
        networkx to be installed. Notice that the edge value must be hashable,
        which is the case for AbstractMatrixGraph subclasses. Edge values are stored 
        under the "value" index. Vertices are stored as indices with a "label"
        value being the corresponding vertex value.

        :returns:  A networkx Graph or DiGraph object.
        """
        try:
            import networkx
        except ImportError:
            raise ImportError, "toNetworkXGraph() requires networkx"

        if self.isUndirected():
            networkXGraph = networkx.Graph()
        else:
            networkXGraph = networkx.DiGraph()

        for i in range(self.getNumVertices()):
            networkXGraph.add_node(i, label=self.getVertex(i))

        allEdges = self.getAllEdges()

        for i in range(allEdges.shape[0]):
            edgeVal = self.getEdge(allEdges[i, 0], allEdges[i, 1])
            networkXGraph.add_edge(allEdges[i, 0], allEdges[i, 1], value=edgeVal)

        return networkXGraph 

    def findConnectedComponents(self):
        """
        Finds a list of all connected components of the graph, in order of size
        with the smallest first.

        :returns: A list of lists of component indices. 
        """
        if not self.isUndirected():
            raise ValueError("Can only find components on undirected graphs")

        vertexIds = set(self.getAllVertexIds())

        components = []

        while len(vertexIds) != 0:
            currentVertex = vertexIds.pop()
            currentComponent = self.depthFirstSearch(currentVertex)
            components.append(currentComponent)
            vertexIds = vertexIds.difference(currentComponent)

        sortedIndices = numpy.array([len(x) for x in components]).argsort()
        sortedComponents = []

        for i in reversed(range(len(components))):
            sortedComponents.append(components[sortedIndices[i]])

        return sortedComponents

    def depthFirstSearch(self, root):
        """
        Depth first search starting from a particular vertex, based on the code found
        in Wikipedia. Returns the set of connected vertices.
        """
        toVisit = set()
        visited = set()

        toVisit.add(root)
        (rows, cols) = numpy.nonzero(self.W)
        edges = numpy.c_[rows, cols]

        while len(toVisit) != 0:
            v = toVisit.pop()

            if v not in visited:
                visited.add(v)

            neighbours = edges[edges[:, 0]==v, 1]
            toVisit = toVisit.union(set(neighbours).difference(visited))

        return visited

==> apgl-0.6.5.tar.gz: apgl-0.6.5/apgl/graph/AbstractSingleGraph.py <==
'''
Created on 30 Jun 2009

@author: charanpal
'''

import numpy 
from apgl.util.Util import Util
from apgl.graph.AbstractGraph import AbstractGraph

class AbstractSingleGraph(AbstractGraph):
    """
    A basic abstract base class for graphs. An AbstractSingleGraph can only have
    a single edge at most between any two vertices, but edges and vertices can be
    labelled with any object. Verticies are indexed with integer values.
    """
    def __init__(self):
        pass

    def addEdge(self, vertexIndex1, vertexIndex2, edgeValue):
        """
        Add a non-zero edge between two vertices.
        
        :param vertexIndex1: The index of the first vertex.
        :type vertexIndex1: :class:`int`

        :param vertexIndex2: The index of the second vertex.
        :type vertexIndex2: :class:`int`

        :param edgeValue: The value to assign to the edge.
        """

        Util.abstract()

    def addEdges(self, edgeIndexArray, edgeValues=[]):
        """
        Takes a numpy array of edge index pairs, and edge values and adds them
        to this graph. The array is 2 dimensional such that each row is a pair
        of edge indices.

        :param edgeIndexArray: The array of edge indices with each being a pair of indices. 
        :type edgeIndexArray: :class:`numpy.ndarray`

        :param edgeValues: The list of edge values
        :type edgeValues: :class:`list`
        """

        if edgeValues != []: 
            for i in range(edgeIndexArray.shape[0]):
                self.addEdge(edgeIndexArray[i, 0], edgeIndexArray[i, 1], edgeValues[i])
        else:
            for i in range(edgeIndexArray.shape[0]):
                self.addEdge(edgeIndexArray[i, 0], edgeIndexArray[i, 1], 1)
    
    def removeEdge(self, vertexIndex1, vertexIndex2):
        """
        Remove an edge between two vertices.

        :param vertexIndex1: The index of the first vertex.
        :type vertexIndex1: :class:`int`

        :param vertexIndex2: The index of the second vertex.
        :type vertexIndex2: :class:`int`
        """
        Util.abstract()
    
    def getNumEdges(self):
        """ Returns the total number of edges in the graph. """
        Util.abstract()
    
    def getNumVertices(self):
        """ Returns the total number of vertices in the graph. """
        Util.abstract()
    
    def isUndirected(self):
        """ Returns true if the current graph is undirected, otherwise false. """
        Util.abstract()
    
    def neighbours(self, vertexIndex1):
        """ Return a iterable item of neighbours (indices) """
        Util.abstract()

    def getEdge(self, vertexIndex1, vertexIndex2):
        """ Return an edge between two vertices """
        Util.abstract()

    def getVertex(self, vertexIndex):
        """ Return a vertex of given index """
        Util.abstract()

    def getVertices(self, vertexIndices):
        """
        Takes a list of vertex indices and returns the corresponding vertex values.

        :param vertexIndices: A list of vertex indices
        :type vertexIndices: :class:`list`

        :returns: A list of vertices corresponding to the indices 
        """
        vertices = [] 

        for vertexIndex in vertexIndices:
            vertices.append(self.getVertex(vertexIndex))

        return vertices 

    def getAllVertexIds(self):
        """
        Return a list of all indices of the vertices
        
        :returns:  A numpy array of all the vertex indices in this graph. 
        """

        Util.abstract()

    def setVertices(self, vertexIndices, vertices):
        """
        Assign new values to the vertices corresponding to vertexIndices

        :param vertexIndices: A list of vertex indices
        :type vertexIndices: :class:`list`

        :param vertices: A list of vertex values
        :type vertices: :class:`list`
        """
        if len(vertexIndices) != len(vertices):
            raise ValueError("Must be lists of same length")

        for i in range(len(vertexIndices)):
            self.setVertex(vertexIndices[i], vertices[i])

    def setVertex(self, vertexIndex, vertex):
        """ Assign a value to a vertex """
        Util.abstract()

    def getAllEdges(self):
        """
        Return an array of edges with each row representing an edge.

        :returns:  A numpy array of all edges in this graph. 
        """
        Util.abstract()

    def toNetworkXGraph(self):
        """
        Convert this graph into a networkx Graph or DiGraph object, which requires 
        networkx to be installed. Notice that the edge value must be hashable,
        which is the case for AbstractMatrixGraph subclasses. Edge values are stored 
        under the "value" index. Vertices are stored as indices with a "label"
        value being the corresponding vertex value.

        :returns:  A networkx Graph or DiGraph object.
        """
        try:
            import networkx
        except ImportError:
            raise ImportError("toNetworkXGraph() requires networkx")

        if self.isUndirected():
            networkXGraph = networkx.Graph()
        else:
            networkXGraph = networkx.DiGraph()

        for i in range(self.getNumVertices()):
            networkXGraph.add_node(i, label=self.getVertex(i))

        allEdges = self.getAllEdges()

        for i in range(allEdges.shape[0]):
            edgeVal = self.getEdge(allEdges[i, 0], allEdges[i, 1])
            networkXGraph.add_edge(allEdges[i, 0], allEdges[i, 1], value=edgeVal)

        return networkXGraph 

    def findConnectedComponents(self):
        """
        Finds a list of all connected components of the graph, in order of size
        with the smallest first.

        :returns: A list of lists of component indices. 
        """
        if not self.isUndirected():
            raise ValueError("Can only find components on undirected graphs")

        vertexIds = set(self.getAllVertexIds())

        components = []

        while len(vertexIds) != 0:
            currentVertex = vertexIds.pop()
            currentComponent = self.depthFirstSearch(currentVertex)
            components.append(currentComponent)
            vertexIds = vertexIds.difference(currentComponent)

        sortedIndices = numpy.array([len(x) for x in components]).argsort()
        sortedComponents = []

        for i in reversed(list(range(len(components)))):
            sortedComponents.append(components[sortedIndices[i]])

        return sortedComponents
